/**********************************************************************

CurrentMonitor.cpp
COPYRIGHT (c) 2013-2016 Gregg E. Berman

Part of DCC++ BASE STATION for the Arduino

**********************************************************************/

// Noter la fonction check() qui utilise maintenant cette signalPin pour couper le courant.

#include "Arduino.h"
#include "DCCpp_Uno.h"
#include "CurrentMonitor.h"
#include "Comm.h"

///////////////////////////////////////////////////////////////////////////////

void CurrentMonitor::begin(int pin, int inSignalPin, const char *msg, float inSampleMax)
{
    this->pin = pin;
    this->signalPin = inSignalPin;
    this->msg = msg;
    this->current = 0;
    this->currentSampleMax = inSampleMax;
} // CurrentMonitor::begin

boolean CurrentMonitor::checkTime()
{
    if(millis( ) - sampleTime < CURRENT_SAMPLE_TIME)       // no need to check current yet
        return(false);
    sampleTime = millis();       // note millis() uses TIMER-0.  For UNO, we change the scale on Timer-0.  For MEGA we do not.  This means CURENT_SAMPLE_TIME is different for UNO then MEGA
    return(true);
} // CurrentMonitor::checkTime

boolean eStop_mem = false;
void CurrentMonitor::check()
{
    if (this->pin == UNDEFINED_PIN || this->signalPin == UNDEFINED_PIN)
        return;

  #if defined(ARDUINO_ARCH_ESP32)
  // Compute an average value of 50 read.
  int base = 0;
	for (int j = 0; j < 50; j++)
	{
		base += analogRead(this->pin);
	}
	this->current = (float) ((base / 50.0)  * 0.9) - 100;
  #else
    this->current = (float)(analogRead(this->pin) * CURRENT_SAMPLE_SMOOTHING + this->current * (1.0 - CURRENT_SAMPLE_SMOOTHING));      // compute new exponentially-smoothed current
  #endif
/*
    int signalPin = DCCppConfig::SignalEnablePinProg;
    if (signalPin == UNDEFINED_PIN)
        signalPin = DCCppConfig::SignalEnablePinMain;
*/
    volatile bool powerState = (digitalRead(signalPin) == HIGH) ? true : false;
    if (powerState && (digitalRead(EmergencyStop) == EmergencyStopState)) {
        delay(100); // read button 2 times 
        if (powerState && (digitalRead(EmergencyStop) == EmergencyStopState)) {
            DCCpp::powerOff();
            DCCPP_INTERFACE.println(F("Arret d'urgence"));
//            DCCPP_INTERFACE.println(F("Emergency Stop"));
#if !defined(USE_ETHERNET)
            DCCPP_INTERFACE.println("");
#endif
        }
    }

#ifdef E_BOOSTER_ENABLE
    boolean eStop = (analogRead(E_BoosterIn) < 800) ? true : false; // low active, for Booster CDE

  #ifdef DCCPP_DEBUG_MODE
    if (eStop != eStop_mem) {
        DCCPP_INTERFACE.print("eStop "); DCCPP_INTERFACE.print(eStop); DCCPP_INTERFACE.print("   "); DCCPP_INTERFACE.println(analogRead(E_BoosterIn));
        DCCPP_INTERFACE.print("powerState "); DCCPP_INTERFACE.println(powerState);
    }
  #endif

    if (eStop && powerState && !eStop_mem) {
        DCCpp::panicStop(eStop);
        DCCPP_INTERFACE.print(F("E_Booster OFF"));

      #if !defined(USE_ETHERNET)
        DCCPP_INTERFACE.println("");
      #endif
    }

    if (!eStop && !powerState && eStop_mem){
        DCCpp::panicStop(eStop);
        DCCPP_INTERFACE.print(F("E_Booster ON "));

      #if !defined(USE_ETHERNET)
        DCCPP_INTERFACE.println("");
      #endif
    }
    eStop_mem = eStop;
#endif

    // current overload processing
    if (this->current > this->currentSampleMax && digitalRead(this->signalPin) == HIGH)
    {
    //  DCCpp::powerOff(); //old method
        digitalWrite(this->signalPin, LOW);
        String origin;
        switch(this->pin) {
          case 54:
            origin = "Main";
            break;
          case 55:
            origin = "Prog";
            break;
          case 56:
            origin = "Ext";
            break;
          case 57:
            origin = "Garage/dépot";
            break;
        }

#ifdef DCCPP_DEBUG_MODE
        DCCPP_INTERFACE.print(origin);
        DCCPP_INTERFACE.println(F(" Power OFF ! Surcharge en courant détectée *****"));
        DCCPP_INTERFACE.print(F("Courant lu : "));
        DCCPP_INTERFACE.println(current);

        DCCPP_INTERFACE.print(this->msg);                  // print corresponding error message
  #if !defined(USE_ETHERNET)
        DCCPP_INTERFACE.println("");
  #endif
#endif
    }
} // CurrentMonitor::check

long int CurrentMonitor::sampleTime=0;
